# coding: utf-8
# © 2015 David BEAL @ Akretion
# © 2015 Sebastien BEAU @ Akretion
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import logging

from openerp.osv import orm
from openerp.tools.translate import _
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT


_logger = logging.getLogger(__name__)


class SaleOrderLine(orm.Model):
    _inherit = 'sale.order.line'

    def distinct_logistic_backend_version(self, cr, uid, ids, context=None):
        """ Check if sale order 'logistics center' version match with
            the product logistics backend version
        """
        line = self.browse(cr, uid, ids, context=context)[0]
        if line.product_id.type == 'service':
            return False
        backend = line.order_id.get_logistic_backend()
        if backend:
            if not line.product_id.logistic_bind_ids:
                return True
            if backend and backend.version:
                for bind in line.product_id.logistic_bind_ids:
                    if (bind.backend_id and bind.backend_id.version and
                            bind.backend_id.version != backend.version):
                        return True
        return False

    def logistic_center_catalog_outofdate(
            self, cr, uid, ids, line, task_method, context=None):
        """Check if product write_date is > last task execution date"""
        backend = line.order_id.get_logistic_backend()
        if backend:
            catalog_task_ids = self.pool['repository.task'].search(cr, uid, [
                ('repository_id', '=', backend.file_repository_id.id),
                ('method', '=', task_method),
            ], context=context)
            assert len(catalog_task_ids) == 1, "Will only take one task id"
            task = self.pool['repository.task'].browse(
                cr, uid, catalog_task_ids[0], context=context)
            if task.last_exe_date and line.product_id:
                # TODO : evaluate if this query must be applied
                # on the whole products of the sale order (-> sale.order)
                query = """
SELECT GREATEST (pt.write_date, pp.write_date) AS update_date
FROM product_template pt LEFT JOIN product_product pp
    ON pp.product_tmpl_id = pt.id
WHERE pp.id = %s """ % line.product_id.id
                cr.execute(query)
                product_date = cr.fetchall()[0][0]
                if product_date > task.last_exe_date:
                    return True
            else:
                return True
            return False
        return False


class SaleOrder(orm.Model):
    _inherit = ['sale.order', 'abstract.logistic.flow']
    _name = 'sale.order'

    def _prepare_order_line_move(self, cr, uid, order, line, picking_id,
                                 date_planned, context=None):
        vals = super(SaleOrder, self)._prepare_order_line_move(
            cr, uid, order, line, picking_id, date_planned, context=context)
        backend = self.get_logistic_backend(
            cr, uid, [order.id], context=context)
        if backend:
            location_id = backend.warehouse_id.lot_stock_id.id
            location_dest_id = backend.warehouse_id.lot_output_id.id
            if location_id and location_dest_id:
                vals['location_id'] = location_id
                vals['location_dest_id'] = location_dest_id
            else:
                raise orm.except_orm(_("Warehouse settings error: "),
                                     _(self.WAREHOUSE_EXCEPTION
                                     % (backend.warehouse_id.name)))
            vals['logistic_center'] = order.logistic_center
        return vals

    def _prepare_order_picking(self, cr, uid, order, context=None):
        vals = super(SaleOrder, self)._prepare_order_picking(
            cr, uid, order, context=context)
        vals['logistic_center'] = order.logistic_center
        return vals

    def onchange_shop_id(self, cr, uid, ids, shop_id, context, partner_id,
                         partner_invoice_id, partner_shipping_id):
        res = super(SaleOrder, self).onchange_shop_id(
            cr, uid, ids, shop_id, context, partner_invoice_id,
            partner_shipping_id)
        if shop_id:
            values = res['value']
            shop = self.pool['sale.shop'].browse(
                cr, uid, shop_id, context=context)
            values.update({'logistic_center': 'internal'})
            if shop.warehouse_id:
                logistics_center = self.get_logistic_backend(
                    cr, uid, [shop.warehouse_id.id], origin='warehouse',
                    context=context)
                res['value'].update({'logistic_center': logistics_center})
        return res
